package com.hai.order.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.security.AESEncodeUtil;
import com.hai.common.security.UserCenter;
import com.hai.common.utils.IDGenerator;
import com.hai.config.HuiLianTongConfig;
import com.hai.config.HuiLianTongUnionCardConfig;
import com.hai.config.JinZhuJiaYouService;
import com.hai.entity.*;
import com.hai.enum_type.GasClassGroupTaskStatus;
import com.hai.enum_type.JinZhuJiaYouOilNo;
import com.hai.enum_type.MerAccountMarketingType;
import com.hai.enum_type.MerchantStoreSourceType;
import com.hai.model.GasPayPriceModel;
import com.hai.model.HighMerchantModel;
import com.hai.model.ResponseData;
import com.hai.order.model.CreateOrderChildModel;
import com.hai.order.service.OrderCreateHandleService;
import com.hai.order.service.OrderService;
import com.hai.order.service.OrderServiceExt;
import com.hai.order.type.OrderChildGoodsType;
import com.hai.order.type.OrderChildStatus;
import com.hai.order.utils.OrderUtil;
import com.hai.service.*;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.net.URLDecoder;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 创建订单-业务校验
 * @className: CreateOrderCheckService
 * @author: HuRui
 * @date: 2022/8/25
 **/
@Service("createOrderHandleService")
public class OrderCreateHandleServiceImpl implements OrderCreateHandleService {

    private static Logger log = LoggerFactory.getLogger(OrderCreateHandleServiceImpl.class);

    @Resource
    private CommonService commonService;

    @Resource
    private OrderService orderService;

    @Resource
    private HighUserService userService;

    @Resource
    private OrderServiceExt orderServiceExt;

    @Resource
    private HighCouponService couponService;

    @Resource
    private HighMerchantStoreService storeService;

    @Resource
    private HighMerchantService merchantService;

    @Resource
    private HighDiscountCouponRelService discountCouponRelService;

    @Resource
    private HighCouponHandselService highCouponHandselService;

    @Resource
    private HighMerchantAccountMarketingService merchantAccountMarketingService;

    @Resource
    private HighGasOrderService gasOrderService;

    @Resource
    private HighGasClassGroupTaskService gasClassGroupTaskService;

    @Resource
    private HighGasStaffService gasStaffService;

    @Resource
    private HighGasDiscountOilPriceService gasDiscountOilPriceService;

    @Resource
    private HighTyAgentOilStationService gasTyAgentOilStationService;

    @Resource
    private JinZhuJiaYouService gasJinZhuJiaYouService;

    @Override
    public List<HighChildOrder> couponHandle(HighDiscountUserRel discountUserRel, CreateOrderChildModel createOrderChild) throws Exception {
            if (createOrderChild.getSaleCount() != 1) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "卡卷只能购买一张");
            }
            // 查询卡卷信息
            HighCoupon coupon = couponService.getCouponDetail(createOrderChild.getGoodsId());
            if (coupon == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.NOT_FOUND_COUPON, "");
            }
            // 是否需要预约
            if (coupon.getReserveStatus() == true) {
                if (createOrderChild.getStoreId() == null) {
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未选择门店");
                }
                // 查询门店信息
                HighMerchantStore merchantStore = storeService.getDetailById(createOrderChild.getStoreId());
                if (merchantStore == null) {
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未找到门店信息");
                }
                if (!merchantStore.getMerchantId().equals(coupon.getMerchantId())) {
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "卡券无法在此门店消费");
                }
            }

            // 是否达到限购数量
            if (orderServiceExt.userBuyCouponNum(createOrderChild.getMemId(), coupon.getId()) > coupon.getLimitNumber()) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, coupon.getCouponName() + ",已达到限购数量");
            }

            // 生成子订单编号
            String childOrderNo = OrderUtil.generateChildOrderNo();

            // 如果不是来源不是贵州中石化，需要校验库存
            if (!coupon.getCouponSource().equals(4)) {
                if (couponService.assignOrderStock(coupon.getId(), childOrderNo) == null) {
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "库存不足");
                }
            }

            List<HighChildOrder> childOrderList = new ArrayList<>();
            HighChildOrder childOrder;
            childOrder = new HighChildOrder();
            childOrder.setMemId(createOrderChild.getMemId());
            childOrder.setMemName(createOrderChild.getMemName());
            childOrder.setMemPhone(createOrderChild.getMemPhone());
            childOrder.setChildOrderNo(childOrderNo);
            childOrder.setGoodsType(OrderChildGoodsType.TYPE1.getNumber());
            childOrder.setGoodsId(coupon.getId());
            childOrder.setGoodsName(coupon.getCouponName());
            childOrder.setGoodsImg(coupon.getCouponImg());
            childOrder.setGoodsSpecName("默认");
            childOrder.setGoodsPrice(coupon.getSalesPrice());
            childOrder.setGoodsActualPrice(coupon.getDiscountPrice());
            childOrder.setSaleCount(createOrderChild.getSaleCount());
            childOrder.setGiveawayType(false);
            childOrder.setChildOrderStatus(OrderChildStatus.STATUS1.getNumber());
            childOrder.setPraiseStatus(0);
            childOrder.setCreateTime(new Date());

            // 贵州中石化
            if (coupon.getCouponSource().equals(4)) {
                // 下单请求
                JSONObject jsonObject = HuiLianTongConfig.recharge(createOrderChild.getOrderNo(),
                        childOrder.getSaleCount(),
                        coupon.getCouponKey(),
                        createOrderChild.getMemPhone());
                if (!jsonObject.getString("respCode").equals("0000")) {
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "下单失败");
                }
                JSONObject response = HuiLianTongUnionCardConfig.resolveResponse(jsonObject.getString("data"));
                if (!response.getString("respCode").equals("0000")) {
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, response.getString("msg"));
                }
                childOrder.setChannelOrderNo(response.getJSONObject("data").getString("rechargeOrderId"));
            }

            // 校验优惠券是否可用  如果使用优惠券就只能使用卡券的原价
            if (discountUserRel != null) {
                List<HighDiscountCouponRel> discountCouponRelList = discountCouponRelService.getRelByCoupon(coupon.getId());
                List<HighDiscountCouponRel> collect = discountCouponRelList.stream().filter(o -> o.getDiscountId().equals(discountUserRel.getDiscountId())).collect(Collectors.toList());
                if (collect == null || collect.size() == 0) {
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "无法使用优惠券");
                }
                childOrder.setGoodsActualPrice(coupon.getSalesPrice());
            }

            childOrder.setTotalPrice(childOrder.getGoodsPrice().multiply(new BigDecimal(childOrder.getSaleCount().toString())));
            childOrder.setTotalActualPrice(childOrder.getGoodsActualPrice().multiply(new BigDecimal(childOrder.getSaleCount().toString())));
            childOrderList.add(childOrder);

            // 是否赠送卡卷
            if (coupon.getIsPresent() == true) {
                // 查询赠送卡券信息
                List<HighCouponHandsel> handselCouponList = highCouponHandselService.getHandselListByCoupon(childOrder.getGoodsId());
                for (HighCouponHandsel handsel : handselCouponList) {
                    // 查询卡卷信息
                    HighCoupon handselCoupon = couponService.getCouponDetail(handsel.getHandselCouponId());
                    if (handselCoupon == null) {
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.NOT_FOUND_COUPON, "");
                    }
                    String handselChildOrderNo = OrderUtil.generateChildOrderNo();

                    // 赠送卡卷是否有库存，没有就不赠送
                    if (couponService.assignOrderStock(handselCoupon.getId(), handselChildOrderNo) != null) {
                        // 生成交易子订单
                        childOrder= new HighChildOrder();
                        childOrder.setMemId(createOrderChild.getMemId());
                        childOrder.setMemName(createOrderChild.getMemName());
                        childOrder.setMemPhone(createOrderChild.getMemPhone());
                        childOrder.setChildOrderNo(handselChildOrderNo);
                        childOrder.setGoodsType(OrderChildGoodsType.TYPE1.getNumber());
                        childOrder.setGoodsId(handselCoupon.getId());
                        childOrder.setGoodsName(handselCoupon.getCouponName());
                        childOrder.setGoodsImg(handselCoupon.getCouponImg());
                        childOrder.setGoodsSpecName("默认");
                        childOrder.setGoodsPrice(new BigDecimal("0"));
                        childOrder.setGoodsActualPrice(new BigDecimal("0"));
                        childOrder.setSaleCount(1);
                        childOrder.setGiveawayType(true);
                        childOrder.setChildOrderStatus(OrderChildStatus.STATUS1.getNumber());
                        childOrder.setPraiseStatus(0);
                        childOrder.setTotalPrice(childOrder.getGoodsPrice().multiply(new BigDecimal(childOrder.getSaleCount().toString())));
                        childOrder.setTotalActualPrice(childOrder.getGoodsActualPrice().multiply(new BigDecimal(childOrder.getSaleCount().toString())));
                        childOrder.setCreateTime(new Date());
                        childOrderList.add(childOrder);
                    }
                }
            }

            return childOrderList;
    }

    @Override
    public HighChildOrder integralRecharge(CreateOrderChildModel createOrderChild) {
        // 查询用户信息
        HighUser user = userService.getDetailDataByUser(createOrderChild.getGoodsId());
        if (user == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未找到用户信息");
        }
        HighChildOrder childOrder = new HighChildOrder();
        childOrder.setChildOrderNo(OrderUtil.generateChildOrderNo());
        childOrder.setGoodsType(OrderChildGoodsType.TYPE1.getNumber());
        childOrder.setGoodsId(user.getId());
        childOrder.setGoodsName(user.getName());
        childOrder.setGoodsImg(user.getHeaderImg());
        childOrder.setGoodsSpecName("默认");
        childOrder.setGoodsPrice(createOrderChild.getGoodsPrice());
        childOrder.setGoodsActualPrice(createOrderChild.getGoodsPrice());
        childOrder.setSaleCount(1);
        childOrder.setTotalPrice(childOrder.getGoodsPrice().multiply(new BigDecimal(childOrder.getSaleCount().toString())));
        childOrder.setTotalActualPrice(childOrder.getGoodsActualPrice().multiply(new BigDecimal(childOrder.getSaleCount().toString())));
        childOrder.setChildOrderStatus(OrderChildStatus.STATUS1.getNumber());
        childOrder.setGiveawayType(false);
        childOrder.setPraiseStatus(0);
        return childOrder;
    }

    @Override
    public HighChildOrder oilHandle(HighOrder order,HighDiscountUserRel discountUserRel, CreateOrderChildModel createOrderChild) throws Exception {
        // 查询油品类型
        SecDictionary gasOilType = commonService.mappingSysCode("GAS_OIL_TYPE", createOrderChild.getGasOilNo());
        if (gasOilType == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知油品类型");
        }
        // 加油限制
        if (gasOilType.getExt1().equals("1")) {
            SecDictionary refuelLimit = commonService.mappingSysName("REFUEL_LIMIT", "汽油");
            if (refuelLimit != null && createOrderChild.getGoodsPrice().compareTo(new BigDecimal(refuelLimit.getCodeValue()))  == 1) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "汽油加油金额不能超过" + refuelLimit.getCodeValue()+"元");
            }
        } else if (gasOilType.getExt1().equals("2")) {
            SecDictionary refuelLimit = commonService.mappingSysName("REFUEL_LIMIT", "柴油");
            if (refuelLimit != null && createOrderChild.getGoodsPrice().compareTo(new BigDecimal(refuelLimit.getCodeValue()))  == 1) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "柴油加油金额不能超过" + refuelLimit.getCodeValue()+"元");
            }
        }
        // 查询油站
        HighMerchantStore store = storeService.getDetailById(createOrderChild.getGoodsId());
        if (store == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知的油站");
        }
        // 查询商户
        HighMerchant merchant = merchantService.getDetailById(store.getMerchantId());
        if (merchant == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知的油站");
        }

        // 营销减免的金额
        BigDecimal marketingPrice = merchantAccountMarketingService.getPrice(store.getMerchantId(), MerAccountMarketingType.type3.getType(), Integer.valueOf(createOrderChild.getGasOilNo()));

        // 计算价格
        GasPayPriceModel priceModel = gasDiscountOilPriceService.oilPriceDiscountCompute(createOrderChild.getGoodsPrice(), createOrderChild.getGoodsId(), createOrderChild.getGasOilNo(), createOrderChild.getIsTyAgent());

        // 生成交易子订单
        HighChildOrder childOrder = new HighChildOrder();
        childOrder.setOrderNo(order.getOrderNo());
        childOrder.setMemId(createOrderChild.getMemId());
        childOrder.setMemName(createOrderChild.getMemName());
        childOrder.setMemPhone(createOrderChild.getMemPhone());
        childOrder.setChildOrderNo(OrderUtil.generateChildOrderNo());
        childOrder.setGoodsType(OrderChildGoodsType.TYPE3.getNumber());
        childOrder.setGoodsId(store.getId());
        childOrder.setGoodsName(store.getStoreName());
        childOrder.setGoodsImg(store.getStoreLogo());
        childOrder.setGoodsSpecName("默认");
        childOrder.setGoodsPrice(priceModel.getOilingPrice());
        childOrder.setGoodsActualPrice(priceModel.getPayPrice());
        childOrder.setSaleCount(createOrderChild.getSaleCount());
        childOrder.setGiveawayType(false);
        childOrder.setChildOrderStatus(OrderChildStatus.STATUS1.getNumber());
        childOrder.setTotalPrice(new BigDecimal(childOrder.getSaleCount().toString()).multiply(childOrder.getGoodsPrice()));
        childOrder.setTotalActualPrice(new BigDecimal(childOrder.getSaleCount().toString()).multiply(childOrder.getGoodsActualPrice()));
        childOrder.setPraiseStatus(0);
        childOrder.setCreateTime(new Date());

        // 生成加油订单
        HighGasOrder gasOrder = new HighGasOrder();
        gasOrder.setChannelType(store.getSourceType());
        gasOrder.setOrderNo(order.getOrderNo());
        gasOrder.setChildOrderNo(childOrder.getChildOrderNo());
        gasOrder.setMemId(childOrder.getMemId());
        gasOrder.setMemPhone(childOrder.getMemPhone());
        gasOrder.setMerId(store.getMerchantId());
        gasOrder.setMerName(merchant.getMerchantName());
        gasOrder.setStoreId(store.getId());
        gasOrder.setStoreName(store.getStoreName());
        gasOrder.setGasRefuelPrice(childOrder.getGoodsPrice());
        gasOrder.setGasOilNo(createOrderChild.getGasOilNo());
        gasOrder.setGasGunNo(createOrderChild.getGasGunNo());
        gasOrder.setGasOilType(Integer.parseInt(gasOilType.getExt1()));
        gasOrder.setStatus(OrderChildStatus.STATUS1.getNumber());

        // 成本价 油站价 减 营销策略价
        gasOrder.setGasPriceCost(priceModel.getPriceGun().subtract(marketingPrice));
        if (marketingPrice.compareTo(new BigDecimal("0")) <= 0) {
            gasOrder.setGasPriceCostTotal(gasOrder.getGasRefuelPrice());
        } else {
            gasOrder.setGasPriceCostTotal(gasOrder.getGasRefuelPrice().multiply(gasOrder.getGasOilLiters()).setScale(2, BigDecimal.ROUND_HALF_DOWN));
        }

        // 使用了优惠价
        if (discountUserRel != null) {
            // 支付价格
            childOrder.setGoodsActualPrice(childOrder.getGoodsPrice());
            // 油价信息
            gasOrder.setGasPriceGun(priceModel.getPriceGun());
            gasOrder.setGasPriceVip(priceModel.getPriceGun());
            gasOrder.setGasPriceOfficial(new BigDecimal("0"));
            gasOrder.setGasPricePlatform(priceModel.getPriceGun());
            gasOrder.setGasOilLiters(priceModel.getOilLiters());
            gasOrder.setGasDiscount(new BigDecimal("0"));
            gasOrder.setGasOilSubsidy(new BigDecimal("0"));
            gasOrder.setGasLitersPreferences(new BigDecimal("0"));
            gasOrder.setGasPricePreferences(priceModel.getPriceGun());
        } else {
            // 支付价格
            childOrder.setGoodsActualPrice(priceModel.getPayPrice());
            // 油价信息
            gasOrder.setGasPriceGun(priceModel.getPriceGun());
            gasOrder.setGasPriceVip(priceModel.getPriceVip());
            gasOrder.setGasPriceOfficial(priceModel.getPriceOfficial());
            gasOrder.setGasPricePlatform(priceModel.getPricePlatform());
            gasOrder.setGasOilLiters(priceModel.getOilLiters());
            gasOrder.setGasDiscount(priceModel.getDiscount());
            gasOrder.setGasOilSubsidy(priceModel.getPreferentialMargin());
            gasOrder.setGasLitersPreferences(priceModel.getLitersPreferences());
            gasOrder.setGasPricePreferences(priceModel.getPricePreferences());
        }

        if (store.getSourceType().equals(MerchantStoreSourceType.type4.getNumber())) {
            // 查询金猪油品数据
            JinZhuJiaYouOilNo jzOilNo = JinZhuJiaYouOilNo.getDataByOilNo(Integer.valueOf(gasOrder.getGasOilNo()));
            if (jzOilNo == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "暂时无法提交订单，未知的油品");
            }
            // 请求金猪接口 - 创建金猪订单
            JSONObject jzOrderObject = gasJinZhuJiaYouService.createOrder(gasOrder.getOrderNo(),
                    gasOrder.getGasRefuelPrice(),
                    store.getStoreKey(),
                    jzOilNo.getOilNoCode(),
                    gasOrder.getGasGunNo(),
                    gasOrder.getMemPhone()
            );
            gasOrder.setChannelOrderNo(jzOrderObject.getJSONObject("data").getString("systemOrderNo"));
            gasOrder.setGasPriceChannelPay(jzOrderObject.getJSONObject("data").getBigDecimal("paymentAmount"));
        }

        // 代理商
        if (createOrderChild.getIsTyAgent() != null && createOrderChild.getIsTyAgent().equals(true)) {
            // 查询加油站
            HighTyAgentOilStation oilStation = gasTyAgentOilStationService.getDetailByOilStationId(createOrderChild.getGoodsId());
            if (oilStation != null) {
                gasOrder.setGasSalesmanId(oilStation.getTySalesmanId());
                gasOrder.setGasSalesmanName(oilStation.getTySalesmanName());
                gasOrder.setGasAgentId(oilStation.getTyAgentId());
                gasOrder.setGasAgentName(oilStation.getTyAgentName());
                gasOrder.setGasOrgId(oilStation.getOrganizationId());
                gasOrder.setGasOrgName(oilStation.getOrganizationName());
            }
        }

        // 加油员
        if (StringUtils.isNotBlank(createOrderChild.getGasStaffCode())) {
            String qrCodeSignKey = commonService.getDictionaryCodeName("QR_CODE_SIGN_KEY", "1");
            if (StringUtils.isBlank(qrCodeSignKey)) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知秘钥");
            }
            String staffId;
            try {
                staffId = AESEncodeUtil.aesDecryptByBytes(AESEncodeUtil.base64Decode(URLDecoder.decode(createOrderChild.getGasStaffCode(),"UTF-8")), qrCodeSignKey);
            } catch (Exception e) {
                log.error("HighOrderController --> addOrder() error!", "加油员code解析失败");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "收银员code解析失败");
            }
            // 查询员工
            HighGasStaff gasStaff = gasStaffService.getStaffDetailById(Long.parseLong(staffId));
            if (gasStaff == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知加油员");
            }
            if (gasStaff.getMerchantStoreId().equals(childOrder.getGoodsId())) {
                gasOrder.setGasStaffId(gasStaff.getId());
                gasOrder.setGasStaffName(gasStaff.getName());
            }
        }

        // 油站是否开启班组
        HighGasClassGroupTask groupTask = gasClassGroupTaskService.getCurrentTaskByStoreId(gasOrder.getStoreId());
        if (groupTask != null) {
            gasOrder.setGasClassGroupId(groupTask.getGasClassGroupId());
            gasOrder.setGasClassGroupName(groupTask.getGasClassGroupName());
            gasOrder.setGasClassGroupTaskId(groupTask.getId());
        }

        gasOrderService.addGasOrder(gasOrder);
        return childOrder;
    }

    @Override
    public ResponseData couponPackage(CreateOrderChildModel createOrderChild) {
        return null;
    }

    @Override
    public ResponseData hltUnionCardRecharge(CreateOrderChildModel createOrderChild) {
        return null;
    }

}
